@@ -37,28 +37,30 @@
  */
 
 #include <stdio.h>
-
 #include <unistd.h>
 #include <sys/socket.h>
 #include <sys/types.h>
-#include <linux/if_alg.h>
 #include <stdint.h>
 #include <errno.h>
 #include <string.h>
 #include <stdlib.h>
+#include <asm/types.h>
+#include <linux/netlink.h>
+#include <linux/rtnetlink.h>
+#include "cryptouser.h"
 
 #include "kcapi.h"
 
 #define MAJVERSION 0 /* API / ABI incompatible changes, functional changes that
 		      * require consumer to be updated */
-#define MINVERSION 2 /* API compatible, ABI may change, functional
+#define MINVERSION 3 /* API compatible, ABI may change, functional
 		      * enhancements only, consumer can be left unchanged if
 		      * enhancements are not considered */
-#define PATCHLEVEL 1 /* API / ABI compatible, no functional changes, no
+#define PATCHLEVEL 0 /* API / ABI compatible, no functional changes, no
 		      * enhancements, bug fixes only */
 
 /* remove once in if_alg.h */
-#define ALG_SET_AEAD_ASSOC		4
+#define ALG_SET_AEAD_ASSOCLEN		4
 #define ALG_SET_AEAD_AUTHSIZE		5
 
 #define ALG_GET_BLOCKSIZE		1
@@ -66,11 +68,6 @@
 #define ALG_GET_AEAD_AUTHSIZE		3
 #define ALG_GET_DIGESTSIZE		4
 
-struct af_alg_aead_assoc {
-	__u32	aead_assoclen;
-	__u8	aead_assoc[0];
-};
-
 /* remove once in socket.h */
 #ifndef AF_ALG
 #define AF_ALG 38
@@ -106,18 +103,17 @@ static ssize_t _kcapi_common_crypt(struct kcapi_handle *handle,
 
 	/* AEAD data */
 	uint32_t *taglen = NULL;
+	uint32_t *assoclen = NULL;
 	size_t taglen_msg_size = handle->aead.taglen ?
 				 CMSG_SPACE(sizeof(*taglen)) : 0;
-	struct af_alg_aead_assoc *alg_assoc = NULL;
-	size_t assoc_msg_size = handle->aead.taglen ?
-				CMSG_SPACE(sizeof(*alg_assoc) +
-					   handle->aead.assoclen) : 0;
+	size_t assoc_msg_size = handle->aead.assoclen ?
+				CMSG_SPACE(sizeof(*assoclen)) : 0;
 
 	size_t bufferlen =
 		CMSG_SPACE(sizeof(*type)) + 	/* Encryption / Decryption */
 		iv_msg_size +			/* IV */
 		taglen_msg_size +		/* AEAD tag length */
-		assoc_msg_size;			/* AEAD associated data */
+		assoc_msg_size;			/* AEAD associated data size */
 
 	memset(&msg, 0, sizeof(msg));
 
@@ -170,18 +166,17 @@ static ssize_t _kcapi_common_crypt(struct kcapi_handle *handle,
 		/* Set associated data length */
 		header = CMSG_NXTHDR(&msg, header);
 		header->cmsg_level = SOL_ALG;
-		header->cmsg_type = ALG_SET_AEAD_ASSOC;
-		header->cmsg_len = assoc_msg_size;
-		alg_assoc = (void*)CMSG_DATA(header);
-		alg_assoc->aead_assoclen = handle->aead.assoclen;
-		memcpy(alg_assoc->aead_assoc, handle->aead.assoc,
-		       handle->aead.assoclen);
+		header->cmsg_type = ALG_SET_AEAD_ASSOCLEN;
+		header->cmsg_len = CMSG_LEN(sizeof(*assoclen));
+		assoclen = (void*)CMSG_DATA(header);
+		*assoclen = handle->aead.assoclen;
 	}
 
 	ret = sendmsg(handle->opfd, &msg, 0);
 	if (ret != (ssize_t)handle->skdata.inlen)
 		goto bad;
 
+	/* TODO: EINTR and ERESTARTSYS */
 	ret = read(handle->opfd, handle->skdata.out, handle->skdata.outlen);
 	if (0 > ret)
 		goto bad;
@@ -246,19 +241,199 @@ static inline int _kcapi_common_setkey(struct kcapi_handle *handle,
 	return 0;
 }
 
-static inline unsigned int _kcapi_common_getinfo(struct kcapi_handle *handle,
-						 int optval)
+#if 0
+static int __kcapi_common_getinfo(struct kcapi_handle *handle,
+				  const char *ciphername,
+				  int drivername)
 {
-	socklen_t len = 0;
-	int outlen = 0;
+	int ret = -EFAULT;
 
-	if (getsockopt(handle->opfd, SOL_ALG, optval, NULL, &len) == -1)
-		return 0;
+	/* NETLINK_CRYPTO specific */
+	char buf[4096];
+	struct nlmsghdr *res_n = (struct nlmsghdr *)buf;
+	struct {
+		struct nlmsghdr n;
+		struct crypto_user_alg cru;
+	} req;
+	struct crypto_user_alg *cru_res;
+	int res_len = 0;
+	struct rtattr *tb[CRYPTOCFGA_MAX+1];
+	struct rtattr *rta;
 
-	outlen = (int)len;
-	if (outlen < 0)
-		outlen = 0;
-	return outlen;
+	/* AunsignedF_NETLINK specific */
+	struct sockaddr_nl nl;
+	int sd = 0;
+	socklen_t addr_len;
+	struct iovec iov;
+	struct msghdr msg;
+
+	memset(&req, 0, sizeof(req));
+	memset(&buf, 0, sizeof(buf));
+	memset(&msg, 0, sizeof(msg));
+
+	req.n.nlmsg_len = NLMSG_LENGTH(sizeof(req.cru));
+	req.n.nlmsg_flags = NLM_F_REQUEST;
+	req.n.nlmsg_type = CRYPTO_MSG_GETALG;
+
+	if (drivername)
+		strncpy(req.cru.cru_driver_name, ciphername,
+			strlen(ciphername));
+	else
+		strncpy(req.cru.cru_name, ciphername, strlen(ciphername));
+
+
+	/* talk to netlink socket */
+	sd =  socket(AF_NETLINK, SOCK_RAW, NETLINK_CRYPTO);
+	if (sd < 0) {
+		perror("Netlink error: cannot open netlink socket");
+		return -EFAULT;
+	}
+	memset(&nl, 0, sizeof(nl));
+	nl.nl_family = AF_NETLINK;
+	if (bind(sd, (struct sockaddr*)&nl, sizeof(nl)) < 0) {
+		perror("Netlink error: cannot bind netlink socket");
+		goto out;
+	}
+	/* sanity check that netlink socket was successfully opened */
+	addr_len = sizeof(nl);
+	if (getsockname(sd, (struct sockaddr*)&nl, &addr_len) < 0) {
+		perror("Netlink error: cannot getsockname");
+		goto out;
+	}
+	if (addr_len != sizeof(nl)) {
+		fprintf(stderr, "Netlink error: wrong address length %d\n",
+			addr_len);
+		goto out;
+	}
+	if (nl.nl_family != AF_NETLINK) {
+		fprintf(stderr, "Netlink error: wrong address family %d\n",
+			nl.nl_family);
+		goto out;
+	}
+
+	/* sending data */
+	iov.iov_base = (void*) &req.n;
+	iov.iov_len = req.n.nlmsg_len;
+	msg.msg_name = &nl;
+	msg.msg_namelen = sizeof(nl);
+	msg.msg_iov = &iov;
+	msg.msg_iovlen = 1;
+	memset(&nl, 0, sizeof(nl));
+	nl.nl_family = AF_NETLINK;
+	if (sendmsg(sd, &msg, 0) < 0) {
+		perror("Netlink error: sendmsg failed");
+		goto out;
+	}
+	memset(buf,0,sizeof(buf));
+	iov.iov_base = buf;
+	while (1) {
+		iov.iov_len = sizeof(buf);
+		ret = recvmsg(sd, &msg, 0);
+		if (ret < 0) {
+			if (errno == EINTR || errno == EAGAIN)
+				continue;
+			perror("Netlink error: netlink receive error");
+			ret = -EFAULT;
+			goto out;
+		}
+		if (ret == 0) {
+			fprintf(stderr, "Netlink error: no data\n");
+			ret = -EFAULT;
+			goto out;
+		}
+		if ((size_t)ret > sizeof(buf)) {
+			perror("Netlink error: received too much data\n");
+			ret = -EFAULT;
+			goto out;
+		}
+		break;
+	}
+
+	ret = -EFAULT;
+	res_len = res_n->nlmsg_len;
+	if (res_n->nlmsg_type == NLMSG_ERROR) {
+		/*
+		 * return -EAGAIN -- this error will occur if we received a
+		 * driver name, but used it for a generic name. Allow caller
+		 * to invoke function again where driver name is looked up
+		 */
+		ret = -EAGAIN;
+		goto out;
+	}
+
+	if (res_n->nlmsg_type == CRYPTO_MSG_GETALG) {
+		cru_res = NLMSG_DATA(res_n);
+		res_len -= NLMSG_SPACE(sizeof(*cru_res));
+	}
+	if (res_len < 0) {
+		fprintf(stderr, "Netlink error: nlmsg len %d\n", res_len);
+		goto out;
+	}
+
+	/* parse data */
+	rta = CR_RTA(cru_res);
+	memset(tb, 0, sizeof(struct rtattr *) * (CRYPTOCFGA_MAX + 1));
+	while (RTA_OK(rta, res_len)) {
+		if ((rta->rta_type <= CRYPTOCFGA_MAX) && (!tb[rta->rta_type]))
+			tb[rta->rta_type] = rta;
+		rta = RTA_NEXT(rta, res_len);
+	}
+	if (res_len) {
+		fprintf(stderr, "Netlink error: unprocessed data %d\n",
+			res_len);
+		goto out;
+	}
+
+	if (tb[CRYPTOCFGA_REPORT_HASH]) {
+		struct rtattr *rta = tb[CRYPTOCFGA_REPORT_HASH];
+		struct crypto_report_hash *rsh =
+			(struct crypto_report_hash *) RTA_DATA(rta);
+		handle->info.hash_digestsize = rsh->digestsize;
+		handle->info.blocksize = rsh->blocksize;
+	}
+	if (tb[CRYPTOCFGA_REPORT_BLKCIPHER]) {
+		struct rtattr *rta = tb[CRYPTOCFGA_REPORT_BLKCIPHER];
+		struct crypto_report_blkcipher *rblk =
+			(struct crypto_report_blkcipher *) RTA_DATA(rta);
+		handle->info.blocksize = rblk->blocksize;
+		handle->info.ivsize = rblk->ivsize;
+		handle->info.blk_min_keysize = rblk->min_keysize;
+		handle->info.blk_max_keysize = rblk->max_keysize;
+	}
+	if (tb[CRYPTOCFGA_REPORT_AEAD]) {
+		struct rtattr *rta = tb[CRYPTOCFGA_REPORT_AEAD];
+		struct crypto_report_aead *raead =
+			(struct crypto_report_aead *) RTA_DATA(rta);
+		handle->info.blocksize = raead->blocksize;
+		handle->info.ivsize = raead->ivsize;
+		handle->info.aead_maxauthsize = raead->maxauthsize;
+	}
+	if (tb[CRYPTOCFGA_REPORT_RNG]) {
+		struct rtattr *rta = tb[CRYPTOCFGA_REPORT_RNG];
+		struct crypto_report_rng *rrng =
+			(struct crypto_report_rng *) RTA_DATA(rta);
+		handle->info.rng_seedsize = rrng->seedsize;
+	}
+
+	ret = 0;
+
+out:
+	close(sd);
+	return ret;
+}
+#endif
+
+static int _kcapi_common_getinfo(struct kcapi_handle *handle,
+				 const char *ciphername)
+{
+	handle->info.blocksize = 16;
+	handle->info.ivsize = 16;
+#if 0
+	int ret = __kcapi_common_getinfo(handle, ciphername, 0);
+	if (ret)
+		return __kcapi_common_getinfo(handle, ciphername, 1);
+	return 0;
+#endif
 }
 
 static int _kcapi_handle_init(struct kcapi_handle *handle,
@@ -275,9 +450,10 @@ static int _kcapi_handle_init(struct kcapi_handle *handle,
 
 	handle->tfmfd = socket(AF_ALG, SOCK_SEQPACKET, 0);
 	if (handle->tfmfd == -1)
-		return -ENOTSUP;
+		return -EOPNOTSUPP;
 
 	if (bind(handle->tfmfd, (struct sockaddr *)&sa, sizeof(sa)) == -1) {
+		perror("AF_ALG: bind failed");
 		close(handle->tfmfd);
 		handle->tfmfd = -1;
 		return -ENOENT;
@@ -285,12 +461,13 @@ static int _kcapi_handle_init(struct kcapi_handle *handle,
 
 	handle->opfd = accept(handle->tfmfd, NULL, 0);
 	if (handle->opfd == -1) {
+		perror("AF_ALG: accept failed");
 		close(handle->tfmfd);
 		handle->tfmfd = -1;
 		return -EINVAL;
 	}
 
-	return 0;
+	return _kcapi_common_getinfo(handle, ciphername);
 }
 
 static inline int _kcapi_handle_destroy(struct kcapi_handle *handle)
@@ -299,6 +476,7 @@ static inline int _kcapi_handle_destroy(struct kcapi_handle *handle)
 		close(handle->tfmfd);
 	if (handle->opfd != -1)
 		close(handle->opfd);
+	memset(handle, 0, sizeof(struct kcapi_handle));
 	return 0;
 }
 
@@ -338,7 +516,7 @@ int kcapi_pad_iv(struct kcapi_handle *handle,
 		 unsigned char **newiv, size_t *newivlen)
 {
 	unsigned char *niv = NULL;
-	unsigned int nivlen = _kcapi_common_getinfo(handle, ALG_GET_IVSIZE);
+	unsigned int nivlen = handle->info.ivsize;
 
 	if (nivlen == ivlen)
 		return -ERANGE;
@@ -369,8 +547,9 @@ int kcapi_pad_iv(struct kcapi_handle *handle,
  * This function provides the initialization of a symmetric cipher handle and
  * establishes the connection to the kernel.
  *
- * Return: 0 upon success; ENOENT - algorithm not available; ENOTSUP - AF_ALG
- *	     family not available; EINVAL - accept syscall failed
+ * Return: 0 upon success; ENOENT - algorithm not available; 
+ *	   -EOPNOTSUPP - AF_ALG family not available;
+ *	   -EINVAL - accept syscall failed
  */
 int kcapi_cipher_init(struct kcapi_handle *handle, const char *ciphername)
 {
@@ -416,7 +595,7 @@ int kcapi_cipher_setkey(struct kcapi_handle *handle,
 int kcapi_cipher_setiv(struct kcapi_handle *handle,
 		       const unsigned char *iv, size_t ivlen)
 {
-	int cipher_ivlen = _kcapi_common_getinfo(handle, ALG_GET_IVSIZE);
+	int cipher_ivlen = handle->info.ivsize;
 
 	if (cipher_ivlen < 0)
 		return cipher_ivlen;
@@ -481,7 +660,7 @@ ssize_t kcapi_cipher_decrypt(struct kcapi_handle *handle,
  */
 int kcapi_cipher_ivsize(struct kcapi_handle *handle)
 {
-	return _kcapi_common_getinfo(handle, ALG_GET_IVSIZE);
+	return handle->info.ivsize;
 }
 
 /**
@@ -492,7 +671,7 @@ int kcapi_cipher_ivsize(struct kcapi_handle *handle)
  */
 int kcapi_cipher_blocksize(struct kcapi_handle *handle)
 {
-	return _kcapi_common_getinfo(handle, ALG_GET_BLOCKSIZE);
+	return handle->info.blocksize;
 }
 
 /**
@@ -517,8 +696,9 @@ int kcapi_cipher_blocksize(struct kcapi_handle *handle)
  * This function initializes an AEAD cipher handle and establishes the
  * connection to the kernel.
  *
- * Return: 0 upon success; -ENOENT - algorithm not available; -ENOTSUP - AF_ALG
- *	   family not available; -EINVAL - accept syscall failed
+ * Return: 0 upon success; -ENOENT - algorithm not available;
+ *	   -EOPNOTSUPP - AF_ALG family not available;
+ *	   -EINVAL - accept syscall failed
  */
 int kcapi_aead_init(struct kcapi_handle *handle, const char *ciphername)
 {
@@ -564,7 +744,7 @@ int kcapi_aead_setkey(struct kcapi_handle *handle,
 int kcapi_aead_setiv(struct kcapi_handle *handle,
 		     const unsigned char *iv, size_t ivlen)
 {
-	int cipher_ivlen = _kcapi_common_getinfo(handle, ALG_GET_IVSIZE);
+	int cipher_ivlen = handle->info.ivsize;
 
 	if (cipher_ivlen < 0)
 		return cipher_ivlen;
@@ -578,17 +758,13 @@ int kcapi_aead_setiv(struct kcapi_handle *handle,
 }
 
 /**
- * kcapi_aead_setassoc() - set associated data for AEAD operation
+ * kcapi_aead_setassoclen() - set associated data for AEAD operation
  * @handle: cipher handle - input
- * @assoc: buffer holding the IV (may be NULL if associated data is not
- *	  needed) - input
  * @assoclen: length of assoc (should be zero if assoc is NULL) - input
  */
-void kcapi_aead_setassoc(struct kcapi_handle *handle,
-			 const unsigned char *assoc, size_t assoclen)
+void kcapi_aead_setassoclen(struct kcapi_handle *handle, size_t assoclen)
 {
-	handle->aead.assoc = assoc;
-	handle->aead.assoclen = assoc ? assoclen : 0;
+	handle->aead.assoclen = assoclen;
 }
 
 /**
@@ -617,7 +793,12 @@ void kcapi_aead_settaglen(struct kcapi_handle *handle, size_t taglen)
  * the plaintext is overwritten with the ciphertext.
  *
  * When using this function, the caller must align the input and output
- * data buffers. The ciphertext buffer must hold the tag as follows:
+ * data buffers. The input buffer must hold the following information:
+ * 	associated authentication data || plaintext
+ * The caller must use kcapi_aead_setassoclen() to specify the size of the
+ * associated data buffer.
+ *
+ * The output ciphertext buffer must hold the tag as follows:
  * 	ciphertext || tag
  * The caller must ensure that the ciphertext buffer is large enough to hold
  * the ciphertext together with the tag of the size set by the caller using
@@ -638,6 +819,8 @@ ssize_t kcapi_aead_encrypt(struct kcapi_handle *handle,
  * @handle: cipher handle - input
  * @pt: plaintext data buffer - input
  * @ptlen: length of plaintext buffer - input
+ * @assoc: associated authentication data buffer - input
+ * @assoclen: length of associated authentication data buffer - input
  * @taglen: length of the authentication tag to be created - input
  *
  * The caller does not need to know anything about the memory structure
@@ -651,22 +834,33 @@ ssize_t kcapi_aead_encrypt(struct kcapi_handle *handle,
  * this function.
  */
 ssize_t kcapi_aead_enc_nonalign(struct kcapi_handle *handle,
-				unsigned char *pt, size_t ptlen, size_t taglen)
+				unsigned char *pt, size_t ptlen,
+				unsigned char *assoc, size_t assoclen,
+				size_t taglen)
 {
-	unsigned char *ctbuf = NULL;
+	unsigned char *ptbuf = NULL;
 
-	if (!ptlen || !taglen)
+	if (!ptlen || !assoclen || !taglen)
 		return -EINVAL;
 
-	ctbuf = calloc(1, ptlen + taglen);
-	if (!ctbuf)
+	/* to perform in-place encryption, have sufficient space */
+	if (assoclen < taglen)
+		ptbuf = calloc(1, ptlen + taglen);
+	else
+		ptbuf = calloc(1, ptlen + assoclen);
+	if (!ptbuf)
 		return -ENOMEM;
 
-	handle->skdata.in = pt;
-	handle->skdata.inlen = ptlen;
-	handle->skdata.out = ctbuf;
+	/* AEAD encryption input buffer layout: assoc data || plaintext */
+	memcpy(ptbuf, assoc, assoclen);
+	memcpy(ptbuf + assoclen, pt, ptlen);
+
+	handle->skdata.in = ptbuf;
+	handle->skdata.inlen = assoclen + ptlen;
+	handle->skdata.out = ptbuf;
 	handle->skdata.outlen = ptlen + taglen;
 	handle->aead.taglen = taglen;
+	handle->aead.assoclen = assoclen;
 
 	return _kcapi_common_crypt(handle, ALG_OP_ENCRYPT);
 }
@@ -704,9 +898,13 @@ void kcapi_aead_enc_getdata(struct kcapi_handle *handle,
 void kcapi_aead_enc_free(struct kcapi_handle *handle)
 {
 	unsigned char *buf = handle->skdata.out;
-	memset(handle->skdata.out, 0, handle->skdata.outlen);
+
+	size_t len = (handle->skdata.inlen < handle->skdata.outlen) ?
+			handle->skdata.outlen : handle->skdata.inlen;
+
+	memset(handle->skdata.out, 0, len);
 	/* magic to convince GCC to memset the buffer */
-	buf = memchr(buf, 1, handle->skdata.outlen);
+	buf = memchr(buf, 1, len);
 	if (buf)
 		buf = '\0';
 	free(handle->skdata.out);
@@ -726,7 +924,7 @@ void kcapi_aead_enc_free(struct kcapi_handle *handle)
  *
  * When using this function, the caller must align the input and output
  * data buffers. The ciphertext buffer must contain the tag as follows:
- * 	ciphertext || tag
+ * 	associated data || ciphertext || tag
  * The plaintext buffer will not hold the tag which means the caller only needs
  * to allocate memory sufficient to hold the plaintext.
  *
@@ -750,6 +948,8 @@ ssize_t kcapi_aead_decrypt(struct kcapi_handle *handle,
  * @handle: cipher handle - input
  * @ct: ciphertext data buffer - input
  * @ctlen: length of ciphertext buffer - input
+ * @assoc: associated authentication data buffer - input
+ * @assoclen: length of associated authentication data buffer - input
  * @tag: authentication tag buffer - input
  * @taglen: length of the authentication tag - input
  *
@@ -765,32 +965,36 @@ ssize_t kcapi_aead_decrypt(struct kcapi_handle *handle,
  */
 ssize_t kcapi_aead_dec_nonalign(struct kcapi_handle *handle,
 				unsigned char *ct, size_t ctlen,
+				unsigned char *assoc, size_t assoclen,
 				unsigned char *tag, size_t taglen)
 {
 	unsigned char *input = NULL;
-	ssize_t ret = 0;
 
-	if (!ctlen || !taglen)
+	if (!ctlen || !assoclen || !taglen)
 		return -EINVAL;
 
 	/* Format input data by concatenating ciphertext and tag */
-	input = calloc(1, ctlen + taglen);
+	input = calloc(1, ctlen + assoclen + taglen);
 	if (!input)
 		return -ENOMEM;
-	memcpy(input, ct, ctlen);
-	memcpy(input + ctlen, tag, taglen);
+
+	/* AEAD encryption input buffer layout: assoc data || plaintext */
+	memcpy(input, assoc, assoclen);
+	memcpy(input + assoclen, ct, ctlen);
+	memcpy(input + assoclen + ctlen, tag, taglen);
 
 	/*
 	 * in-place decryption as ciphertext buffer is larger than plaintext
 	 * buffer
 	 */
 	handle->skdata.in = input;
-	handle->skdata.inlen = ctlen + taglen;
+	handle->skdata.inlen = assoclen + ctlen + taglen;
 	handle->skdata.out = input;
 	handle->skdata.outlen = ctlen;
 	handle->aead.taglen = taglen;
-	ret = _kcapi_common_crypt(handle, ALG_OP_DECRYPT);
-	return ret;
+	handle->aead.assoclen = assoclen;
+
+	return _kcapi_common_crypt(handle, ALG_OP_DECRYPT);
 }
 
 /**
@@ -840,7 +1044,7 @@ void kcapi_aead_dec_free(struct kcapi_handle *handle)
  */
 int kcapi_aead_ivsize(struct kcapi_handle *handle)
 {
-	return _kcapi_common_getinfo(handle, ALG_GET_IVSIZE);
+	return handle->info.ivsize;
 }
 
 /**
@@ -851,7 +1055,7 @@ int kcapi_aead_ivsize(struct kcapi_handle *handle)
  */
 int kcapi_aead_blocksize(struct kcapi_handle *handle)
 {
-	return _kcapi_common_getinfo(handle, ALG_GET_BLOCKSIZE);
+	return handle->info.blocksize;
 }
 
 /**
@@ -866,7 +1070,7 @@ int kcapi_aead_blocksize(struct kcapi_handle *handle)
  */
 int kcapi_aead_authsize(struct kcapi_handle *handle)
 {
-	return _kcapi_common_getinfo(handle, ALG_GET_AEAD_AUTHSIZE);
+	return handle->info.aead_maxauthsize;
 }
 
 /**
@@ -916,8 +1120,9 @@ int kcapi_aead_ccm_nonce_to_iv(const unsigned char *nonce, size_t noncelen,
  * This function provides the initialization of a (keyed) message digest handle
  * and establishes the connection to the kernel.
  *
- * Return: 0 upon success; ENOENT - algorithm not available; ENOTSUP - AF_ALG
- *	     family not available; EINVAL - accept syscall failed
+ * Return: 0 upon success; ENOENT - algorithm not available;
+ *	   -EOPNOTSUPP - AF_ALG family not available;
+ *	   -EINVAL - accept syscall failed
  */
 int kcapi_md_init(struct kcapi_handle *handle, const char *ciphername)
 {
@@ -1020,7 +1225,7 @@ ssize_t kcapi_md_final(struct kcapi_handle *handle,
  */
 int kcapi_md_digestsize(struct kcapi_handle *handle)
 {
-	return _kcapi_common_getinfo(handle, ALG_GET_DIGESTSIZE);
+	return handle->info.hash_digestsize;
 }
 
 /**
@@ -1036,8 +1241,9 @@ int kcapi_md_digestsize(struct kcapi_handle *handle)
  * This function provides the initialization of a random number generator handle
  * and establishes the connection to the kernel.
  *
- * Return: 0 upon success; ENOENT - algorithm not available; ENOTSUP - AF_ALG
- *	     family not available; EINVAL - accept syscall failed
+ * Return: 0 upon success; ENOENT - algorithm not available; 
+ *	   -EOPNOTSUPP - AF_ALG family not available;
+ *	   -EINVAL - accept syscall failed
  */
 int kcapi_rng_init(struct kcapi_handle *handle, const char *ciphername)
 {
