@@ -37,6 +37,34 @@
 #ifndef _KCAPI_H
 #define _KCAPI_H
 
+#include <linux/if_alg.h>
+
+/**
+ * Information obtained for different ciphers during handle init time
+ * using the NETLINK_CRYPTO interface.
+ * @blocksize block size of cipher (hash, symmetric, AEAD)
+ * @ivsize size of IV of cipher (symmetric, AEAD)
+ * @hash_digestsize size of message digest (hash)
+ * @blk_min_keysize minimum key size (symmetric)
+ * @blk_max_keysize maximum key size (symmetric)
+ * @aead_maxauthsize maximum authentication tag size (AEAD)
+ * @rng_seedsize seed size (RNG)
+ */
+struct kcapi_cipher_info {
+	/* generic */
+	int blocksize;
+	int ivsize;
+	/* hash */
+	int hash_digestsize;
+	/* blkcipher */
+	int blk_min_keysize;
+	int blk_max_keysize;
+	/* aead */
+	int aead_maxauthsize;
+	/* rng */
+	int rng_seedsize;
+};
+
 /**
  * Common data required for symmetric and AEAD ciphers
  * @in: Input data (plaintext for encryption, ciphertext for
@@ -59,14 +87,12 @@ struct kcapi_skcipher_data {
 
 /**
  * AEAD data
- * @assoc: Associated data - input
  * @assoclen: Length of associated data - input
  * @taglen: Length of authentication tag - input
  * @tag: Authentication tag - input for decryption, output for encryption
  * @retlen: internal data -- number of bytes returned by the read system call
  */
 struct kcapi_aead_data {
-	const unsigned char *assoc;
 	size_t assoclen;
 	size_t taglen;
 	unsigned char *tag;
@@ -85,6 +111,7 @@ struct kcapi_handle {
 	int opfd;
 	struct kcapi_skcipher_data skdata;
 	struct kcapi_aead_data aead;
+	struct kcapi_cipher_info info;
 };
 
 int kcapi_cipher_init(struct kcapi_handle *handle, const char *ciphername);
@@ -108,14 +135,15 @@ int kcapi_aead_setkey(struct kcapi_handle *handle,
 		      const unsigned char *key, size_t keylen);
 int kcapi_aead_setiv(struct kcapi_handle *handle,
 		     const unsigned char *iv, size_t ivlen);
-void kcapi_aead_setassoc(struct kcapi_handle *handle,
-			 const unsigned char *assoc, size_t assoclen);
+void kcapi_aead_setassoclen(struct kcapi_handle *handle, size_t assoclen);
 void kcapi_aead_settaglen(struct kcapi_handle *handle, size_t taglen);
 ssize_t kcapi_aead_encrypt(struct kcapi_handle *handle,
 			   const unsigned char *in, size_t inlen,
 			   unsigned char *out, size_t outlen);
 ssize_t kcapi_aead_enc_nonalign(struct kcapi_handle *handle,
-				unsigned char *pt, size_t ptlen, size_t taglen);
+				unsigned char *pt, size_t ptlen,
+				unsigned char *assoc, size_t assoclen,
+				size_t taglen);
 void kcapi_aead_enc_getdata(struct kcapi_handle *handle,
 			    unsigned char **ct, size_t *ctlen,
 			    unsigned char **tag, size_t *taglen);
@@ -125,6 +153,7 @@ ssize_t kcapi_aead_decrypt(struct kcapi_handle *handle,
 			   unsigned char *out, size_t outlen);
 ssize_t kcapi_aead_dec_nonalign(struct kcapi_handle *handle,
 				unsigned char *ct, size_t ctlen,
+				unsigned char *assoc, size_t assoclen,
 				unsigned char *tag, size_t taglen);
 void kcapi_aead_dec_getdata(struct kcapi_handle *handle,
 			    unsigned char **pt, size_t *ptlen);
